#include "mp3_module.h"

/* ------------------------------------------------------------------- MACROS */



/* ---------------------------------------------------------------- VARIABLES */

static module_t         mp3_module;

#if (_MMB_MODULE_MP3 == 1)

    /**
     * MP3 player specific variables
     */
    static mp3_player_state_t   mp3_state;
    static uint8_t              mp3_volume;
    static mp3_playlist_t       mp3_playlist[3];
    static mp3_song_number_t    mp3_song_number;

#endif

#if (_MMB_MODULE_MICROSD == 1)

    /**
     * Micro SD specific variables
     */
    static int8_t      sd_err;
    static int8_t      sd_fd;
    static uint32_t    sd_seek;
    static sd_file_t   sd_file;

#endif

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */

/**
 * This function executes if player is in running state
 * This function opens SD card if SD card is closed
 * This function plays tracks if SD card is opened
 */
static void player_running ( void );

/**
 * This function executes if player is in stopped state
 * This function closes SD card if it was opened
 * This function does nothing if SD card is closed
 */
static void player_stopped ( void );

/* ----------------------------------------------------------- IMPLEMENTATION */

/**
 * This function increases mp3 player volume on Left and Right speaker
 */
void mp3_increase_volume( void )
{
    uint16_t aux_volume;

    if (mp3_volume > 0x00)
    {
        mp3_volume--;

        aux_volume = 0x00;
        aux_volume |= mp3_volume;
        aux_volume <<= 8;
        aux_volume |= mp3_volume;

        bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, aux_volume );
    }
}

/**
 * This function decreases mp3 player volume on Left and Right speaker
 */
void mp3_decrease_volume( void )
{
    uint16_t aux_volume;

    if (mp3_volume < 0xFD)
    {
        mp3_volume++;

        aux_volume = 0x00;
        aux_volume |= mp3_volume;
        aux_volume <<= 8;
        aux_volume |= mp3_volume;

        bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, aux_volume );
    }
}

/**
 * This function mutes mp3 player by setting its volume to min value (0xFD)
 * This function unmutes mp3 player by setting its volume to max value (0x00)
 * First call to this function mutes mp3 player
 * Second call to this function unmutes mp3 player
 */
void mp3_mute_volume( void )
{
    static uint8_t aux_flag = 0;
    uint16_t aux_volume;

    switch (aux_flag)
    {
        case 0 :
        {
            bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, 0xFDFD );

            aux_flag = 1;

            break;
        }
        case 1 :
        {
            aux_volume = 0x00;
            aux_volume |= mp3_volume;
            aux_volume <<= 8;
            aux_volume |= mp3_volume;

            bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR, aux_volume );

            aux_flag = 0;

            break;
        }
        default :
        {
            break;
        }
    }
}

/**
 * This function stops mp3 player and sets new mp3 song number
 * This function increases song number
 * This function sets song number to first song if current song number is last song
 */
void mp3_next_song( void )
{
    /**
     * Stop mp3 player
     */
    player_stopped( );

    switch (mp3_song_number)
    {
        case SONG_1:
        {
            mp3_song_number = SONG_2;
            break;
        }
        case SONG_2:
        {
            mp3_song_number = SONG_3;
            break;
        }
        case SONG_3:
        {
            mp3_song_number = SONG_1;
            break;
        }
        default :
        {
            /**
             * Error
             */
            break;
        }
    }

    /**
     * Set SD file size to 0
     * Set SD seek variable to 0
     */
    sd_file.sd_file_size = 0;
    sd_seek = 0;
}

/**
 * This function stops mp3 player and sets new mp3 song number
 * This function decreases song number
 * This function sets song number to last song if current song number is first song
 */
void mp3_prev_song( void )
{
    /**
     * Stop mp3 player
     */
    player_stopped( );

    switch (mp3_song_number)
    {
        case SONG_1:
        {
            mp3_song_number = SONG_3;
            break;
        }
        case SONG_2:
        {
            mp3_song_number = SONG_1;
            break;
        }
        case SONG_3:
        {
            mp3_song_number = SONG_2;
            break;
        }
        default :
        {
            /**
             * Error
             */
            break;
        }
    }

    /**
     * Set SD file size to 0
     * Set SD seek variable to 0
     */
    sd_file.sd_file_size = 0;
    sd_seek = 0;
}

/**
 * This function changes mp3 player state from playing to stopped and vice versa
 */
void mp3_module_state_changed ( void )
{
    switch (mp3_state)
    {
        case MP3_PLAY:
        {
            mp3_state = MP3_STOP;
            break;
        }
        case MP3_STOP:
        {
            mp3_state = MP3_PLAY;
            break;
        }
        default :
        {
            /**
             * Error
             */
            while(1);
        }
    }
}

/**
 * This function initializes mp3 module
 * This function is called from system initialization procedure
 */
void mp3_module_init ( module_t ** module )
{
    mp3_module.state = MODULE_INACTIVE;

    #if (_MMB_MODULE_MP3 == 1)

        /**
         * Initialize mp3 driver
         */
        bsp_mp3_init( );

        /**
         * Configure mp3 device
         */
        bsp_mp3_cmd_write( _BSP_MP3_MODE_ADDR,   0x0800 );
        bsp_mp3_cmd_write( _BSP_MP3_BASS_ADDR,   0x7A00 );
        bsp_mp3_cmd_write( _BSP_MP3_CLOCKF_ADDR, 0xC000 );
        Delay_ms(100);
        bsp_mp3_cmd_write( _BSP_MP3_VOL_ADDR,    0x0000 );
        
        /**
         * Set mp3 volume to max
         */
        mp3_volume      = 0x00;
        mp3_song_number = SONG_1;

        /**
         * Initialize mp3 playlist
         */
        strncpy( mp3_playlist[ SONG_1 ].SONG, "song_1.mp3", 11 );
        strncpy( mp3_playlist[ SONG_2 ].SONG, "song_2.mp3", 11 );
        strncpy( mp3_playlist[ SONG_3 ].SONG, "song_3.mp3", 11 );

    #endif

    #if (_MMB_MODULE_MICROSD == 1)

        /**
         * Set SD file size to 0
         * Set SD file state to closed
         */
        sd_file.sd_file_size = 0;
        sd_file.sd_file_state = SD_FILE_CLOSED;

    #endif

    /**
     * Initialize mp3 module
     */
    *module = &mp3_module;
}

void mp3_module_task ( void )
{
    switch ( mp3_module.state )
    {
        case MODULE_INACTIVE :
        {
            /**
             * Draw mp3 screen elements
             */
            DrawImage( &Image_00_03_MP3 );
            DrawButton( &Button_00_04_PLAY );
            DrawButton( &Button_00_05_MUTE );
            DrawButton( &Button_00_06_VOL_INC );
            DrawButton( &Button_00_07_VOL_DEC );
            DrawButton( &Button_00_08_NEXT );
            DrawButton( &Button_00_09_PREV );

            /**
             * Set mp3 module state to active
             * Set mp3 state to stopped
             */
            mp3_module.state = MODULE_ACTIVE;
            mp3_state = MP3_STOP;

            break;
        }
        case MODULE_ACTIVE :
        {
            switch ( mp3_state )
            {
                case MP3_STOP :
                {
                    player_stopped( );

                    break;
                }
                case MP3_PLAY :
                {
                    player_running( );

                    break;
                }
                default :
                {
                    /**
                     * Error
                     */
                    while (1);
                }
            }

            break;
        }
        default :
        {
            /**
             * Error
             */
            while (1);
        }
    }
}

/* --------------------------------------------- PRIVATE FUNCTION DEFINITIONS */

static void player_running ( void )
{
    static uint8_t i;

    switch (sd_file.sd_file_state)
    {
        case SD_FILE_CLOSED:
        {
            sd_fd = 0;

            /**
             * Open file from playlist
             */
            if (( sd_fd = FAT32_Open( mp3_playlist[ mp3_song_number ].SONG, FILE_READ ) ) >= 0)
            {
                /**
                 * Success
                 */
            }
            else
            {
                /**
                 * Error
                 */
                while (1);
            }

            /**
             * Get opened file size [in bytes]
             */
            if (( sd_err = FAT32_Size( mp3_playlist[ mp3_song_number ].SONG, &sd_file.sd_file_size ) ) >= 0)
            {
                /**
                 * Success
                 */
                sd_seek = 0; /** This variable is used with 'FAT32_Seek()' function to seek last played byte from SD file */
            }
            else
            {
                /**
                 * Error
                 */
                while (1);
            }

            /**
             * Set SD file state to open
             */
            sd_file.sd_file_state = SD_FILE_OPEN;

            break;
        }
        case SD_FILE_OPEN:
        {
            if (sd_file.sd_file_size > 32)
            {
                /**
                 * Read 32 bytes from file if file size if greather than 32 bytes
                 */
                if (( sd_err = FAT32_Read( sd_fd, sd_file.sd_file_data, 32 ) ) >= 0)
                {
                    for (i = 0; i < 32; i ++)
                    {
                        /**
                         * Play 32 bytes on mp3 player
                         */
                        while ( bsp_mp3_data_write( sd_file.sd_file_data[ i ] ) );
                    }

                    /**
                     * Decrease file size for 32 bytes that were sent to mp3 player
                     * Increase seek variable for 32 bytes in order to find next bytes to play
                     */
                    sd_file.sd_file_size -= 32;
                    sd_seek += 32;

                    /**
                     * Seek next bytes to play
                     */
                    FAT32_Seek( sd_fd, sd_seek );
                }
                else
                {
                    /**
                     * Error
                     */
                    while (1);
                }
            }
            else if (sd_file.sd_file_size > 0)
            {
                /**
                 * Read all bytes from file if file size is greater than 0 and less than 32 [bytes]
                 */
                if (( sd_err = FAT32_Read( sd_fd, sd_file.sd_file_data, sd_file.sd_file_size ) ) >= 0)
                {
                    /**
                     * Play remaining bytes on mp3 player
                     */
                    for (i = 0; i < sd_file.sd_file_size; i ++)
                    {
                        while ( bsp_mp3_data_write( sd_file.sd_file_data[ i ] ) );
                    }

                    /**
                     * Set SD file size to 0
                     * Set SD seek variable to 0
                     */
                    sd_file.sd_file_size = 0;
                    sd_seek = 0;
                }
                else
                {
                    /**
                     * Error
                     */
                    while (1);
                }
            }
            else
            {
                switch (sd_file.sd_file_state)
                {
                    case SD_FILE_CLOSED:
                    {
                        /**
                         * Error
                         */
                        while (1);
                    }
                    case SD_FILE_OPEN:
                    {
                        /**
                         * Close SD file
                         */
                        if (( sd_err = FAT32_Close( sd_fd ) ) >= 0)
                        {
                            /**
                             * Success
                             */
                        }
                        else
                        {
                            /**
                             * Error
                             */
                            while (1);
                        }

                        /**
                         * Set DS file state to closed
                         */
                        sd_file.sd_file_state = SD_FILE_CLOSED;

                        break;
                    }
                    default :
                    {
                        /**
                         * Error
                         */
                        while (1);
                    }
                }
            }

            break;
        }
        default :
        {
            /**
             * Error
             */
            while (1);
        }
    }
}

static void player_stopped ( void )
{
    switch (sd_file.sd_file_state)
    {
        case SD_FILE_CLOSED:
        {
            /**
             * Not an error - do nothing
             */
            break;
        }
        case SD_FILE_OPEN:
        {
            /**
             * Close SD file if it was opened
             */
            if (( sd_err = FAT32_Close( sd_fd ) ) >= 0)
            {
                sd_file.sd_file_state = SD_FILE_CLOSED;

                sd_file.sd_file_size = 0;
                sd_seek = 0;
            }
            else
            {
                /**
                 * Error
                 */
                while (1);
            }

            break;
        }
        default :
        {
            /**
             * Error
             */
            while (1);
        }
    }
}