/*
    bsp_flash_driver.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.

  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

#include "hal.h"
#include "bsp_flash_driver.h"

#if ( _MMB_MODULE_FLASH == 1 )

/* ------------------------------------------------------------------- MACROS */

#define START_PAGE_ADDRESS               0x010000
#define END_PAGE_ADDRESS                 0x7FFFFF

//  Read status register macros

#define FLASH_BUSY              ( bsp_flash_get_status( ) & 0x01 )
#define FLASH_WEL               ( bsp_flash_get_status( ) & 0x02 )
#define FLASH_WSE               ( bsp_flash_get_status( ) & 0x04 )
#define FLASH_WSP               ( bsp_flash_get_status( ) & 0x08 )
#define FLASH_WPLD              ( bsp_flash_get_status( ) & 0x10 )

//  CE functions. Delay is necessary for higher SPI speed.

#define CE_LO                   BSP_FLASH_CE = 0; Delay_1us( )
#define CE_HI                   Delay_1us( ); BSP_FLASH_CE = 1

//  Timeout macros. Use it to configure watchdog.

#define SECTOR_ERASE_TIMEOUT            10000
#define BLOCK_ERASE_TIMEOUT             4000
#define CHIP_ERASE_TIMEOUT              2000
#define PAGE_WRITE_TIMEOUT              2000
#define FLASH_INIT_TIMEOUT              2000

/* -------------------------------------------- PRIVATE FUNCTION DECLARATIONS */

/*  
    Converts address to array.
    Also checks does address is in proper range.
*/

static int address_convert ( uint32_t address, uint8_t * arr );

/*  
    Waits desired number of cycler for flash to become available.
    Function can be adjusted to desired time period using internal delay call.
*/

static int wait_while_busy ( uint32_t cyc );

/* ----------------------------------------------------------- IMPLEMENTATION */

void bsp_flash_instruction ( uint8_t instruction )
{
    uint8_t tmp = instruction;

    CE_LO;
    hal_spi_write( &tmp, 1 );
    CE_HI;
}

void bsp_flash_set_config ( uint8_t cfg )
{
    uint8_t tmp[3]; 

    tmp[0] = _BSP_FLASH_WRSR;
    tmp[1] = 0;
    tmp[2] = cfg;

    bsp_flash_instruction( _BSP_FLASH_WREN );

    CE_LO;
    hal_spi_write( tmp, 3 );
    CE_HI;
}

uint8_t bsp_flash_get_config ( void )
{
    uint8_t tmp = _BSP_FLASH_RDCR;

    CE_LO;
    hal_spi_write( &tmp, 1 );
    hal_spi_read( &tmp, 1 );
    CE_HI;

    return tmp;
}

uint8_t bsp_flash_get_status ( void )
{
    uint8_t tmp = _BSP_FLASH_RDSR;

    CE_LO;
    hal_spi_write( &tmp, 1 );
    hal_spi_read( &tmp, 1 );
    CE_HI;

    return tmp;
}

flash_rc_t bsp_flash_init ( void )
{
    flash_rc_t rc;

    hal_spi_map( &BSP_FLASH_SPI_MODULE );
    hal_flash_init( );

    /*
        Initialization procedure :

            - Reset flash 
            - Global unlock
    */

    bsp_flash_instruction( _BSP_FLASH_RSTEN );
    bsp_flash_instruction( _BSP_FLASH_RST );
    bsp_flash_instruction( _BSP_FLASH_WREN );
    bsp_flash_instruction( _BSP_FLASH_ULBPR );

    rc =  wait_while_busy( FLASH_INIT_TIMEOUT );

    return rc;
}

flash_rc_t bsp_flash_sector_erase ( uint32_t address )
{
    uint8_t     tmp[4];
    flash_rc_t  rc;

    tmp[0] = _BSP_FLASH_SE;
    tmp[1] = 0;
    tmp[2] = 0;
    tmp[3] = 0;

    if ( ( rc = address_convert( address, &tmp[1] ) ) == FLASH_OK )
    {
        bsp_flash_instruction( _BSP_FLASH_WREN );        

        CE_LO;
        hal_spi_write( tmp, 4 );        
        CE_HI;

        rc =  wait_while_busy( SECTOR_ERASE_TIMEOUT );
    }

    return rc;
}

flash_rc_t bsp_flash_block_erase ( uint32_t address )
{
    uint8_t     tmp[4];
    flash_rc_t  rc;

    tmp[0] = _BSP_FLASH_BE;
    tmp[1] = 0;
    tmp[2] = 0;
    tmp[3] = 0;

    if ( ( rc = address_convert( address, &tmp[1] ) ) == FLASH_OK )
    {
        bsp_flash_instruction( _BSP_FLASH_WREN );        

        CE_LO;
        hal_spi_write( tmp, 4 );        
        CE_HI;

        rc =  wait_while_busy( BLOCK_ERASE_TIMEOUT );
    }

    return rc;
}

flash_rc_t bsp_flash_chip_erase ( void )
{
    flash_rc_t rc;

    bsp_flash_instruction( _BSP_FLASH_WREN );
    bsp_flash_instruction( _BSP_FLASH_CE );

    rc =  wait_while_busy( CHIP_ERASE_TIMEOUT );

    return rc;
}

flash_rc_t bsp_flash_write ( uint32_t address, uint8_t * _data, uint8_t len )
{
    uint8_t     tmp[4];
    flash_rc_t  rc;

    tmp[0] = _BSP_FLASH_PP;
    tmp[1] = 0;
    tmp[2] = 0;
    tmp[3] = 0;

    if ( ( rc = address_convert( address, &tmp[1] ) ) == FLASH_OK )
    {
        bsp_flash_instruction( _BSP_FLASH_WREN );        

        CE_LO;
        hal_spi_write( tmp, 4 );                                                //  Write instruction and address
        hal_spi_write( _data, ( uint16_t ) len );                               //  Write data.
        CE_HI;

        rc =  wait_while_busy( PAGE_WRITE_TIMEOUT );
    }

    return rc;
}

flash_rc_t bsp_flash_read ( uint32_t address, uint8_t * _data, uint8_t len )
{
    uint8_t     tmp[4];
    flash_rc_t  rc;

    tmp[0] = _BSP_FLASH_READ;
    tmp[1] = 0;
    tmp[2] = 0;
    tmp[3] = 0;

    if ( ( rc = address_convert( address, &tmp[1] ) ) == FLASH_OK )
    {
        CE_LO;
        hal_spi_write( tmp, 4 );                                                //  Write instruction and address
        hal_spi_read( _data, ( uint16_t ) len );                                //  Read data.
        CE_HI;
    }
    
    return rc;
}

/* --------------------------------------------- PRIVATE FUNCTION DEFINITIONS */

static int address_convert ( uint32_t address, uint8_t * arr )
{    
    if ( ( address < START_PAGE_ADDRESS ) || ( address > END_PAGE_ADDRESS ) )
    {
        return FLASH_ERROR;
    }

    arr[0] = ( uint8_t ) ( ( address & 0x00FF0000 ) >> 16 );
    arr[1] = ( uint8_t ) ( ( address & 0x0000FF00 ) >> 8 );
    arr[2] = ( uint8_t )   ( address & 0x000000FF );

    return FLASH_OK;
}

static int wait_while_busy ( uint32_t cyc )
{
    uint32_t wd = 0;

    while ( FLASH_BUSY )
    {
        //  Replace this function call with desired cycle to adjust watchdog.

        Delay_1ms( );

        if ( ++wd > cyc )
        {
            //  Watchdog expired. 

            return FLASH_ERROR;
        }
    }

    return FLASH_OK;
}

#endif

/* -------------------------------------------------------------------------- */
/*
    bsp_flash_driver.c

  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektronika.

4. Neither the name of the MikroElektronika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */