/*
    microsd_test.h

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   microsd_test.c

- Version               :   1.0.0
- Date                  :   June 2019
- Developer             :   Marko Jovanovic

*/
/* -------------------------------------------------------------------------- */

#include "test_types.h"
#include "test_console.h"

// Include BSP drivers which will be tested.
//  ...

#include "bsp_microsd_driver.h"

#if ( _MMB_MODULE_MICROSD == 1 )

//  Default task delay in ms

#define TASK_DELAY                      5000
#define TASK_NAME                       "MICROSD Test"

//  Declare necessary initialization variables.
//  ...

static uint32_t WRITE_LEN            = 13;
static uint8_t  FILE_NAME[10]        = "Test.txt";
static uint8_t  WRITE_DATA_BUF[20]       = "MicroSD Test";

static int      err;
static int      fd;
static uint8_t  read_data[20];
static uint32_t file_size;

//  ...

void microsd_test ( test_param_t * tp )
{
    switch ( tp->step ) 
    {
        case TEST_PREINIT :
        {
            tp->step = TEST_INIT;
            tp->delay = TASK_DELAY;
            strcpy( tp->name, TASK_NAME );

            break;
        }
        case TEST_INIT :
        {
            //  Initialize BSP module.
            //  ...

            bsp_microsd_init( );

            //  Perform all necessary post initialization and configuration actions.
            //  ...

            if ( ( err = FAT32_Init( ) ) < 0 )
            {
                //  Retry initialization each second.

                test_console_write_line( "FAT32 Initialization Error" );

                return;
            }

            //  Navigate to the root directory.

            if ( ( err = FAT32_ChangeDir( "\\" ) ) < 0 )
            {
                test_console_write_line( "FAT32 Navigation Error" );

                return;
            }

            //  ...
            //  End of module initialization.


            tp->step = TEST_TASK;

            break;
        }
        case TEST_TASK :
        {
            uint16_t i;
            uint16_t j;

            //  Perform all necessary test tasks.
            //  ...

            test_console_operation( 1, "File Open" );

            if ( ( fd = FAT32_Open( FILE_NAME, FILE_WRITE ) ) >= 0 )
            {
                test_console_output( "File name", ( void * ) FILE_NAME, STRING_FORMAT, "\r\n" );
            }
            else
            {
                test_console_write_line( "FAT32 Open Error" );
            }

            if ( ( err = FAT32_Size( FILE_NAME, &file_size ) ) >= 0 )
            {
                test_console_output( "File size", ( void * ) &file_size, UNSIGNED_LONG_FORMAT , "" );
            }
            else
            {
                test_console_write_line( "FAT32 Size Error" );
            }
            Delay_ms( 1000 );

            test_console_write_line( NULL );
            test_console_operation( 2, "File Write" );


            if ( ( err = FAT32_Write( fd, WRITE_DATA_BUF, WRITE_LEN ) ) >= 0 )
            {
                test_console_output( "Data", ( void * ) &WRITE_DATA_BUF, STRING_FORMAT, "\r\n" );
                test_console_output( "Size", ( void * ) &WRITE_LEN, UNSIGNED_LONG_FORMAT, "" );
            }
            else
            {
                test_console_write_line( "FAT32 Size Error" );
            }
            Delay_ms( 1000 );

            test_console_write_line( NULL );
            test_console_operation( 3, "File Close" );

            if ( ( err = FAT32_Close( fd ) ) >= 0 )
            {
                test_console_output( "File closed", ( void * ) FILE_NAME, STRING_FORMAT, "" );
            }
            else
            {
                test_console_write_line( "FAT32 Close Error" );
            }
            Delay_ms( 1000 );

            test_console_write_line( NULL );
            test_console_operation( 4, "File Reopen" );

            fd = 0;

            if ( ( fd = FAT32_Open( FILE_NAME, FILE_READ ) ) >= 0 )
            {
                test_console_output( "File name", ( void * ) FILE_NAME, STRING_FORMAT, "\r\n" );
            }
            else
            {
                test_console_write_line( "FAT32 Open Error" );
            }

            if ( ( err = FAT32_Size( FILE_NAME, &file_size ) ) >= 0 )
            {
                test_console_output( "File size", ( void * ) &file_size, UNSIGNED_LONG_FORMAT, "" );
            }
            else
            {
                test_console_write_line( "FAT32 Size Error" );
            }
            Delay_ms( 1000 );

            test_console_write_line( NULL );
            test_console_operation( 6, "File Delete" );

            if ( ( err = FAT32_Close( fd ) ) >= 0 )
            {
                test_console_output( "File closed", ( void * ) FILE_NAME, STRING_FORMAT, "\r\n" );
            }
            else
            {
                test_console_write_line( "FAT32 Close Error" );
            }
            Delay_ms( 1000 );

            FAT32_Delete( FILE_NAME );

            if ( FAT32_Exists( FILE_NAME ) == 0 )
            {
                test_console_output( "File deleted", ( void * ) FILE_NAME, STRING_FORMAT, "\r\n" );
            }
            else
            {
                test_console_write_line( "Delete Error" );
            }

            //  ...
            //  End of test task.

            break;
        }
    }
} 

#endif

/* -------------------------------------------------------------------------- */
/*
    microsd_test.c

  Copyright (c) 2017, MikroElektronika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the MikroElektronika.

4. Neither the name of the MikroElektronika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */