/*
    test_runner.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.
  
  Copyright (c) 2017, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

/**
        \file   test_runner.c

- Version               :   1.0.1
- Date                  :   Jul 2019
- Developer             :   Milos Vidojevic

---

    Common test runner.

---

v1.0.0 - Module created
v1.0.1 - USB test procedure preprocessor involved


*/
/* -------------------------------------------------------------------------- */

#include "bsp.h"
#include "test_types.h"
#include "test_console.h"

/* ------------------------------------------------------------------- MACROS */

/*
    Test Runner Macro.

    Za svaku mikromediju je uveden poseban test za USB host, a poseban test za 
    sve ostale module usled nemogucnosti da se PLLovi podese da u istom trenutku 
    budu odgovarajuci i za USB i za ETH.

    Potrebno je odkomentarisati makro kod generisanja hex-a za test proceduru 
    USB hosta dok je za test proceduru svih ostalih modula na sistemo potrebno 
    zakomentarisati makro.

    Takodje je jako bitno loadovati odgovarajucu clock shemu.
*/

#ifndef __KINETIS__
//    #define _USB_TEST_
#endif

/* -------------------------------------------------------- PRIVATE FUNCTIONS */

static void run_single_test ( test_procedure f )
{
    test_param_t tp = TEST_PARAM_DEFAULT;
    
RESTART :
    
    tp.step = TEST_PREINIT;

    //  Test pre initialization.

    f( &tp );
    test_console_header( tp.name );
    test_console_sub_header( tp.name, "Initializing" );

    //  Test initialization.
    
    f( &tp );
    test_console_sub_header( tp.name, "Task Started" );

    for ( ; ; )
    {
        uint8_t  rc;
        uint16_t i;

        //  Test task.

        f( &tp );

#ifndef _USB_TEST_

        for ( i = 0; i < tp.delay; ++i )
        {
            if ( ( rc = test_console_read( ) ) != 0 )
            {
                switch ( rc )
                {
                    case '-' :            tp.delay -= 100;     break;
                    case '+' :            tp.delay += 100;     break;
                    case 'r' : case 'R' :                      goto RESTART;
                    case 'n' : case 'N' :                      goto NEXT;
                    default  :                                 break;
                }
            }

            Delay_1ms( );
        }

#endif

    }
    
NEXT :

    test_console_sub_header( tp.name, "Finished" );

    return;
} 

extern void ui_test ( test_param_t * tp );

#if ( _MMB_MODULE_POWER == 1 )

extern void power_test ( test_param_t * tp );

#endif
#if ( _MMB_MODULE_MICROSD == 1 )

extern void microsd_test ( test_param_t * tp );

#endif
#if ( _MMB_MODULE_FLASH == 1 )

extern void flash_test ( test_param_t * tp );

#endif
#if ( _MMB_MODULE_MP3 == 1 )

extern void mp3_test ( test_param_t * tp );

#endif
#if ( _MMB_MODULE_ACCEL == 1 )

extern void accel_test ( test_param_t * tp );

#endif
#if ( _MMB_MODULE_USB == 1 )

extern void usb_test ( test_param_t * tp );

#endif

/* --------------------------------------------------------- PUBLIC FUNCTIONS */

void test_runner ( void )
{

#ifndef _USB_TEST_

    test_sequence_start( );

#endif
#ifdef _USB_TEST_
#   if ( _MMB_MODULE_USB == 1 )
                                            run_single_test( usb_test );
#   endif
#else
#   if ( _MMB_MODULE_MP3 == 1 )
                                            run_single_test( mp3_test );
#   endif
#   if ( _MMB_MODULE_ACCEL == 1 )
                                            run_single_test( accel_test );
#   endif
#   if ( _MMB_MODULE_POWER == 1 )
                                            run_single_test( power_test );
#   endif
#   if ( _MMB_MODULE_MICROSD == 1 )
                                            run_single_test( microsd_test );
#   endif
#   if ( _MMB_MODULE_FLASH == 1 )
                                            run_single_test( flash_test );
#   endif
#endif
                                            run_single_test( ui_test );
    test_sequence_finish( );

    return;
}


/* -------------------------------------------------------------------------- */
/*
    test_runner.c

  Copyright (c) 2017,  MikroElektronika - http://www.mikroe.com

  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

3. All advertising materials mentioning features or use of this software
   must display the following acknowledgment:
   This product includes software developed by the  MikroElektronika.

4. Neither the name of the  MikroElektronika nor the
   names of its contributors may be used to endorse or promote products
   derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY MIKROELEKTRONIKA ''AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MIKROELEKTRONIKA BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

----------------------------------------------------------------------------- */